@@ -46,11 +46,17 @@ static bool on_focus_out (GtkWidget *widget, GdkEventFocus *event, gcpTools *too
 	tools->Focus (false);
 	return false;
 }
+static bool on_delete_event (GtkWidget* widget, GdkEvent *event, gpointer data)
+{
+	// The toolbox must not be closed
+	return true;
+}
 
 gcpTools::gcpTools (Application *App): Dialog (App, DATADIR"/gchempaint-unstable/ui/tools.glade", "tools")
 {
 	g_signal_connect (G_OBJECT (dialog), "focus_in_event", G_CALLBACK (on_focus_in), this);
 	g_signal_connect (G_OBJECT (dialog), "focus_out_event", G_CALLBACK (on_focus_out), this);
+	g_signal_connect (G_OBJECT (dialog), "delete-event", G_CALLBACK (on_delete_event), NULL);
 	gcpApplication *pApp = dynamic_cast<gcpApplication*> (App);
 	pApp->SetDialog ("Tools", this);
 	m_UIManager = NULL;
