@@ -4,7 +4,7 @@
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,6 +30,7 @@
 #include "application.h"
 #include "tool.h"
 #include "text.h"
+#include "theme.h"
 #include "libgcpcanvas/gprintable.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include <math.h>
@@ -40,6 +41,8 @@
 #include <locale.h>
 #include <fstream>
 
+extern bool new_ui;
+
 /*
 Derivation of a new widget from gnome_canvas with an event for updating canvas size
 */
@@ -166,49 +169,33 @@ gcpView::gcpView (gcpDocument *pDoc, bool Embedded)
 {
 	m_pDoc = pDoc;
 	m_PangoFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoFontDesc, DefaultFontFamily);
-	pango_font_description_set_style (m_PangoFontDesc, DefaultFontStyle);
-	pango_font_description_set_weight (m_PangoFontDesc, DefaultFontWeight);
-	pango_font_description_set_variant (m_PangoFontDesc, DefaultFontVariant);
-	pango_font_description_set_stretch (m_PangoFontDesc, DefaultFontStretch);
-	pango_font_description_set_size (m_PangoFontDesc, DefaultFontSize);
+	pango_font_description_set_family (m_PangoFontDesc, pDoc->GetFontFamily ());
+	pango_font_description_set_style (m_PangoFontDesc, pDoc->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoFontDesc, pDoc->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoFontDesc, pDoc->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoFontDesc, pDoc->GetFontStretch ());
+	pango_font_description_set_size (m_PangoFontDesc, pDoc->GetFontSize ());
 	m_sFontName = pango_font_description_to_string (m_PangoFontDesc);
 	m_PangoSmallFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoSmallFontDesc, DefaultFontFamily);
-	pango_font_description_set_style (m_PangoSmallFontDesc, DefaultFontStyle);
-	pango_font_description_set_weight (m_PangoSmallFontDesc, DefaultFontWeight);
-	pango_font_description_set_variant (m_PangoSmallFontDesc, DefaultFontVariant);
-	pango_font_description_set_stretch (m_PangoSmallFontDesc, DefaultFontStretch);
-	pango_font_description_set_size (m_PangoSmallFontDesc, DefaultFontSize * 2 / 3);
+	pango_font_description_set_family (m_PangoSmallFontDesc, pDoc->GetFontFamily ());
+	pango_font_description_set_style (m_PangoSmallFontDesc, pDoc->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoSmallFontDesc, pDoc->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoSmallFontDesc, pDoc->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoSmallFontDesc, pDoc->GetFontStretch ());
+	pango_font_description_set_size (m_PangoSmallFontDesc, pDoc->GetFontSize () * 2 / 3);
 	m_sSmallFontName = pango_font_description_to_string (m_PangoSmallFontDesc);
 	m_PangoTinyFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoTinyFontDesc, DefaultFontFamily);
-	pango_font_description_set_style (m_PangoTinyFontDesc, DefaultFontStyle);
-	pango_font_description_set_weight (m_PangoTinyFontDesc, DefaultFontWeight);
-	pango_font_description_set_variant (m_PangoTinyFontDesc, DefaultFontVariant);
-	pango_font_description_set_stretch (m_PangoTinyFontDesc, DefaultFontStretch);
-	pango_font_description_set_size (m_PangoTinyFontDesc, DefaultFontSize / 2);
+	pango_font_description_set_family (m_PangoTinyFontDesc, pDoc->GetFontFamily ());
+	pango_font_description_set_style (m_PangoTinyFontDesc, pDoc->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoTinyFontDesc, pDoc->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoTinyFontDesc, pDoc->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoTinyFontDesc, pDoc->GetFontStretch ());
+	pango_font_description_set_size (m_PangoTinyFontDesc, pDoc->GetFontSize () / 2);
 	m_sTinyFontName = pango_font_description_to_string (m_PangoTinyFontDesc);
-	m_DefaultPadding = DefaultPadding;
-	m_DefaultStereoBondWidth = DefaultStereoBondWidth;
-	m_DefaultHashWidth = DefaultHashWidth;
-	m_DefaultHashDist = DefaultHashDist;
-	m_DefaultBondWidth = DefaultBondWidth;
-	m_DefaultBondLength = DefaultBondLength;
-	m_DefaultBondDist = DefaultBondDist;
-	m_DefaultArrowLength = DefaultArrowLength;
-	m_DefaultArrowHeadA = DefaultArrowHeadA;
-	m_DefaultArrowHeadB = DefaultArrowHeadB;
-	m_DefaultArrowHeadC = DefaultArrowHeadC;
-	m_DefaultArrowDist = DefaultArrowDist;
-	m_DefaultObjectPadding = DefaultObjectPadding;
-	m_DefaultSignPadding = DefaultSignPadding;
-	m_ArrowPadding = DefaultArrowPadding;
-	m_StoichiometryPadding = DefaultStoichiometryPadding;
 	m_width = 400;
 	m_height = 300;
 	m_ActiveRichText = NULL;
-	if (!(m_bEmbedded = Embedded))
+	if (!(m_bEmbedded = Embedded) && !new_ui)
 		CreateNewWidget ();
 	m_UIManager = gtk_ui_manager_new ();
 	m_Dragging = false;
@@ -229,8 +216,11 @@ gcpView::~gcpView ()
 
 bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 {
-	gcpTool* pActiveTool = m_pDoc->GetApplication()->GetActiveTool();
-	if ((!m_pDoc->GetEditable()) || (!pActiveTool)) return true;
+	gcpApplication *App = m_pDoc->GetApplication ();
+	gcpTheme *pTheme = m_pDoc->GetTheme ();
+	gcpTool* pActiveTool = App? App->GetActiveTool (): NULL;
+	if ((!m_pDoc->GetEditable ()) || (!pActiveTool))
+		return true;
 	if (item == (GnomeCanvasItem*)m_ActiveRichText)
 	{
 		GnomeCanvasItemClass* klass = GNOME_CANVAS_ITEM_CLASS(((GTypeInstance*)item)->g_class);
@@ -255,7 +245,7 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 				if ((*i).first->GetType() == BondType)
 				{
 					pBond = (gcpBond*)(*i).first;
-					if (pBond->GetDist(x / m_pData->ZoomFactor, y / m_pData->ZoomFactor) < (m_DefaultPadding + m_DefaultBondWidth / 2) / m_pData->ZoomFactor)
+					if (pBond->GetDist(x / pTheme->GetZoomFactor (), y / pTheme->GetZoomFactor ()) < (pTheme->GetPadding () + pTheme->GetBondWidth () / 2) / pTheme->GetZoomFactor ())
 					{
 						item = GNOME_CANVAS_ITEM((*i).second);
 						break;
@@ -351,21 +341,6 @@ GtkWidget* gcpView::CreateNewWidget()
 		m_pData->View = this;
 		gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(m_pWidget), 1);
 		gnome_canvas_set_scroll_region(GNOME_CANVAS(m_pWidget), 0, 0, m_width, m_height);
-		m_pData->ZoomFactor = DefaultZoomFactor;
-		m_pData->Padding = m_DefaultPadding * DefaultZoomFactor;
-		m_pData->BondWidth = m_DefaultBondWidth * DefaultZoomFactor;
-		m_pData->BondLength = m_DefaultBondLength * DefaultZoomFactor;
-		m_pData->BondDist = m_DefaultBondDist * DefaultZoomFactor;
-		m_pData->ArrowLength = m_DefaultArrowLength * DefaultZoomFactor;
-		m_pData->ArrowHeadA = m_DefaultArrowHeadA * DefaultZoomFactor;
-		m_pData->ArrowHeadB = m_DefaultArrowHeadB * DefaultZoomFactor;
-		m_pData->ArrowHeadC = m_DefaultArrowHeadC * DefaultZoomFactor;
-		m_pData->ArrowDist = m_DefaultArrowDist * DefaultZoomFactor;
-		m_pData->HashWidth = m_DefaultHashWidth * DefaultZoomFactor;
-		m_pData->HashDist = m_DefaultHashDist * DefaultZoomFactor;
-		m_pData->StereoBondWidth = m_DefaultStereoBondWidth * DefaultZoomFactor;
-		m_pData->ObjectPadding = m_DefaultObjectPadding * DefaultZoomFactor;
-		m_pData->SignPadding = m_DefaultSignPadding * DefaultZoomFactor;
 		m_pData->Zoom = 1.0;
 		m_pData->Background = gnome_canvas_item_new(
 									gnome_canvas_root(GNOME_CANVAS(m_pWidget)),
@@ -380,7 +355,8 @@ GtkWidget* gcpView::CreateNewWidget()
 									gnome_canvas_root(GNOME_CANVAS(m_pWidget)),
 									gnome_canvas_group_ext_get_type (),
 									NULL));
-		g_signal_connect(G_OBJECT(m_pData->Background), "event", G_CALLBACK(on_event), m_pWidget);
+		if (m_pDoc->GetEditable ())
+			g_signal_connect(G_OBJECT(m_pData->Background), "event", G_CALLBACK(on_event), m_pWidget);
 		g_signal_connect(G_OBJECT(m_pWidget), "destroy", G_CALLBACK(on_destroy), this);
 		g_signal_connect(G_OBJECT(m_pWidget), "size_allocate", G_CALLBACK(on_size), this);
 		g_signal_connect(G_OBJECT(m_pWidget), "realize", G_CALLBACK(gnome_canvas_gcp_update_bounds), this);
@@ -398,7 +374,7 @@ GtkWidget* gcpView::CreateNewWidget()
 			m_PangoContext = gtk_widget_create_pango_context(m_pWidget);
 			g_object_ref(G_OBJECT(m_PangoContext));
 			// bad hack to get charge sign size.
-			if (ChargeSignSize == 0.) {
+/*			if (DefaultChargeSignSize == 0.) {
 				PangoLayout *pl = pango_layout_new (m_PangoContext);
 				char *markup = g_strdup_printf ("<span font_desc=\"%s\">\xE2\x8a\x95</span>", m_sSmallFontName);
 				pango_layout_set_markup (pl, markup, -1);
@@ -406,12 +382,12 @@ GtkWidget* gcpView::CreateNewWidget()
 				PangoLayoutIter* iter = pango_layout_get_iter (pl);
 				PangoRectangle ink, log;
 				pango_layout_iter_get_run_extents (iter, &ink, &log);
-				ChargeYAlign = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+				DefaultChargeYAlign = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 				pango_layout_iter_free (iter);
 				g_object_unref (pl);
-				ChargeSignSize = ink.width / PANGO_SCALE;
-				ChargeSignHeight = log.height / PANGO_SCALE;
-			}
+				DefaultChargeSignSize = ink.width / PANGO_SCALE;
+				DefaultChargeSignHeight = log.height / PANGO_SCALE;
+			}*/
 			UpdateFont();
 		}
 	}
@@ -476,7 +452,7 @@ GnomeCanvasItem* gcpView::GetBackground()
 
 double gcpView::GetZoomFactor()
 {
-	return m_pData->ZoomFactor;
+	return m_pDoc->GetTheme ()->GetZoomFactor ();
 }
 
 void gcpView::UpdateFont()
@@ -491,7 +467,7 @@ void gcpView::UpdateFont()
 	pl = pango_layout_new(m_PangoContext);
 	pango_layout_set_text(pl, "C", 1);
 	pango_layout_get_extents(pl, &rect, NULL);
-	m_BaseLineOffset =  (double(rect.height / PANGO_SCALE) / 2.0) / DefaultZoomFactor;
+	m_BaseLineOffset =  (double(rect.height / PANGO_SCALE) / 2.0) / m_pDoc->GetTheme ()->GetZoomFactor ();
 	g_object_unref(G_OBJECT(pl));
 }
 	
@@ -793,8 +769,8 @@ void gcpView::UpdateSize(double x1, double y1, double x2, double y2)
 	{
 		x1 = - x1;
 		y1 = - y1;
-		gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(m_pWidget), "data");
-		m_pDoc->Move(x1 / pData->ZoomFactor, y1 / pData->ZoomFactor);
+		gcpTheme* pTheme = m_pDoc->GetTheme ();
+		m_pDoc->Move(x1 / pTheme->GetZoomFactor (), y1 / pTheme->GetZoomFactor ());
 		Update(m_pDoc);
 	}
 }
@@ -1009,7 +985,7 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		xmlNewDtd (doc,
 			   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
 			   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
-		doc->children = xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL);
+		xmlDocSetRootElement (doc, xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL));
 		xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
 		xmlSetNs (doc->children, ns);
 		xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
